import os
import threading
import time
import socket
import uuid
import shutil
import requests
import urllib
from urllib import parse, request
from PyQt5.QtCore import QSettings
from kylindbus.enums import LOG_PATH, KYLIN_SOFTWARE_PROPERTIES_LOG, gZipFile, PINGBACK_INTERNET_FILE_URL, \
    PINGBACK_INTRANET_FILE_URL, \
    PINGBACK_INTRANET_URL, PINGBACK_INTERNET_URL
from kylindbus.log_manager import logger

import sys
import dbus
import json
import configparser
import hashlib
import time
import datetime
import base64
from binascii import a2b_hex
from Crypto.PublicKey import RSA
from Crypto.Cipher import PKCS1_OAEP


class ConnectServerThread(threading.Thread):
    def __init__(self, dbusDaemon):
        threading.Thread.__init__(self)
        self.dbusDaemon = dbusDaemon
        self.appname = ""
        self.appversion = ""
        self.state = ""
        self.errorcode = ""

    #
    # 函数：检查是否有上次未发送成功的数据，重新发送
    #
    def resend_info_failed(self):

        ret = os.popen("who | awk '{print $1}'").readlines()
        username = ""
        if ret:
            username = ret[0].strip()

        send_fail_list = []
        if  os.path.exists("/usr/share/kylin-update-manager/kum-url-fail.list"):
            f = open("/usr/share/kylin-update-manager/kum-url-fail.list", encoding='utf-8')
            while True:
                line = f.readline()
                if line:
                    try:
                        req = urllib.request.urlopen(url=line.replace(" ","_"),timeout=3)
                        ret = req.read().decode("utf-8")
                        # 访问成功，但是没有插入数据库
                        if req.status != 200  or "insert success" not in ret:
                            send_fail_list.append(line.strip())
                    except Exception as e:
                        # 如果访问失败，记录访问失败数据，下次启动时再去发送数据
                        send_fail_list.append(line.strip())
                else:
                    break
            f.close()
        filename = "/home/" + username + "/.cache/kylin-update-desktop/kum-url-fail.list"
        if  os.path.exists(filename):
            f = open(filename, encoding='utf-8')
            while True:
                line = f.readline()
                if line:
                    try:
                        req = urllib.request.urlopen(url=line.replace(" ","_"), timeout=3)
                        ret = req.read().decode("utf-8")
                        # 访问成功，但是没有插入数据库
                        if req.status != 200  or "insert success" not in ret:
                            send_fail_list.append(line.replace(" ",""))
                    except Exception as e:
                        # 如果访问失败，记录访问失败数据，下次启动时再去发送数据
                        send_fail_list.append(line.strip())
                else:
                    break
            f.close()
        if len(send_fail_list) != 0:
            # 将这次发送失败的继续写入文件
            f = open("/usr/share/kylin-update-manager/kum-url-fail.list", "wb+")
            f.truncate()
            for str in send_fail_list:
                f.write(str.encode(encoding = "utf-8"))
            f.close()
        else:
            os.system("rm /usr/share/kylin-update-manager/kum-url-fail.list")
            os.system("rm " + filename)        

    def get_values(self, _appname="", _appversion="", _state='', _errorcode=''):
        self.appname = _appname
        self.appversion = _appversion
        self.state = _state
        self.errorcode = _errorcode

    def run(self):
        host_ip, version, soft_version, host_mac = '', '', '', ''
        # 获取本机ip
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect(('8.8.8.8', 80))
            host_ip = s.getsockname()[0]
        except:
            host_ip = 'Ip failed to get'
        # 获取系统版本
        with open('/etc/lsb-release', 'r') as f:
            for line in f.readlines():
                if line.strip().startswith('DISTRIB_DESCRIPTION='):
                    versions = line.strip().split('=')
                    if "V10" in line and "SP1" in versions[1]:
                        version = "V10SP1"
                    else:
                        version = "V10Pro"
                    break
        # 获取软件版本
        output = os.popen('dmidecode -s system-serial-number').readlines()
        if output:
            soft_version = output[0].strip()
        # 获取时间
        nowtime = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))
        # 获取Mac
        mac = os.popen("cat /sys/class/net/$(ip route show default | awk '/default/ {print $5}')/address").readlines()
        if mac:
            hostmac = mac[0].strip()
        # 获取序列号
        if os.path.exists("/etc/.kyinfo"):
            settings = QSettings("/etc/.kyinfo", QSettings.IniFormat)
            settings.beginGroup("servicekey")
            key = str(settings.value("key")).strip()
            settings.endGroup()
        else:
            key = "0"
        try:
            # 用于收集源管理器的更新日志
            if self.errorcode == "10000049" or self.errorcode == "10000005":
                log_dir = os.path.join(LOG_PATH, host_mac + "_" + nowtime)
                log_file_gzip = log_dir + ".tar.gz"
                os.makedirs(log_dir, exist_ok=True)
                if os.path.exists(KYLIN_SOFTWARE_PROPERTIES_LOG):
                    with open(os.path.join(log_dir, "apt_list"), "w") as f:
                        for file in os.listdir("/var/lib/apt/lists"):
                            f.write(file + "\n")
                    shutil.copy(KYLIN_SOFTWARE_PROPERTIES_LOG, log_dir)
                gZipFile(log_dir, log_file_gzip)
                header = {'Content-Type': "multipart/form-data", "Accept-Encoding": "gzip"}
                try:
                    with open(log_file_gzip, "rb") as f:
                        requests.post(PINGBACK_INTRANET_FILE_URL + "filename=" + os.path.basename(log_file_gzip),
                                      data=f.read(), headers=header)
                except:
                    with open(log_file_gzip, "rb") as f:
                        requests.post(PINGBACK_INTERNET_FILE_URL + "filename=" + os.path.basename(log_file_gzip),
                                      data=f.read(), headers=header)
                shutil.rmtree(log_dir)
                os.remove(log_file_gzip)
            else:
                log_file_gzip = ""
            kmg = parse.urlencode(
                {'ip': host_ip, 'version': version, 'soft_version': soft_version, 'datetime': nowtime,
                 'host_mac': host_mac, 'appname': self.appname, 'appversion': self.appversion, 'serial_number': key,
                 'status': self.state, 'filename': log_file_gzip, 'errorcode': self.errorcode})
            url_tmp = ""
            # 优先使用内网服务器，再使用外网
            try:
                url = PINGBACK_INTRANET_URL + kmg
                url_tmp = url
                req = urllib.request.urlopen(url=url, timeout=3)
                ret = req.read().decode("utf-8")
                if req.status != 200  or "insert success" not in ret:
                    f = open("/usr/share/kylin-update-manager/kum-url-fail.list", "a+")
                    if url_tmp not in f.read().decode('utf-8'):
                        f.write(url_tmp + "\n")
                    f.close()
                logger.info("成功访问局域网日志服务器")
            except:
                url = PINGBACK_INTERNET_URL + kmg
                req = urllib.request.urlopen(url=url, timeout=3)
                ret = req.read().decode("utf-8")
                if req.status != 200  or "insert success" not in ret:
                    f = open("/usr/share/kylin-update-manager/kum-url-fail.list", "a+")
                    if url_tmp not in f.read().decode('utf-8'):
                        f.write(url_tmp + "\n")
                    f.close()
                logger.info("成功访问外网日志服务器")
        except Exception as e:
            f = open("/usr/share/kylin-update-manager/kum-url-fail.list", "a+")
            str_tmp = f.read()
            if url_tmp not in str_tmp:
                f.write(url_tmp + "\n")
            f.close()
            logger.error("访问日志服务器失败")

        #发送到daq

        #获取当前时间
        # UTC时间
        utc_time = datetime.datetime.utcnow()
        # 转时间字符串
        utc_time = utc_time.strftime("%Y-%m-%d %H:%M:%S.%f")[:-3]
        time_suffix = utc_time.split(".")[1]
        # 字符串转时间元祖
        utc_time = time.strptime(utc_time, "%Y-%m-%d %H:%M:%S.%f")
        # 时间元祖转时间戳
        utc_time = time.mktime(utc_time)
        # 生成东八区时间时间戳
        now_time = utc_time + 8*60*60
        # 时间戳转时间元祖
        now_time = time.localtime(now_time)
        # 时间元祖转字符串
        now_time = time.strftime("%Y-%m-%d %H:%M:%S",now_time)
        now_time = now_time + "." +time_suffix

        sendInfoJson={"SN":soft_version,"appname":self.appname,"updateTime":nowtime,"appversion":self.appversion,"serial_number":key,"status":self.state,"createTimeStamp":now_time}
        sendInfo=json.dumps(sendInfoJson)

        #构建packageInfo
        tid=""
        conf = configparser.ConfigParser()
        conf.read("/usr/share/kylin-update-manager/tid.conf")
        sections = conf.sections()
        if "uploadMessage" in sections:
            tid = conf.get('uploadMessage','tid')

        packageInfoJson={"packageName":"kylin-system-updater","messageType":"kylin-update-manager","tid":tid}
        packageInfo=json.dumps(packageInfoJson)


        #获取sha256的值
        hsobj = hashlib.sha256()
        try:
            hsobj.update(sendInfo.encode("utf-8"))
        except ValueError as e:
            logger.error("SHA256 value error: %s.",str(e))
        except Exception as e:
            logger.error(str(e))

        key = "LS0tLS1CRUdJTiBQVUJMSUMgS0VZLS0tLS0KTUlJQklqQU5CZ2txaGtpRzl3MEJBUUVGQUFPQ0FR\
    OEFNSUlCQ2dLQ0FRRUFzdW1NTFJEdlFNb0tEQkRJODRqSgpqc1A0Mk55V0pWVEZob2Jra3ZiT05j\
    dExYTXVzRmo2TzJUblZYU3Z6VlVLSjRqZkpwT2l2WEphOVB5Z2wzYTRnClBzSU40enNCMEdOY0tr\
    R3VsS2RrV2x6S3lWQ2xlTzhiQnN6SjkwbTc3cWF6YWg3a1A0TUl0WTVFczBpSkpiR0oKN1MxcERj\
    MlJkNnVFQWJLaXJyRTFlNzlFTEd4am5VN2V5NWkyRDE2WWJoZEQwZ2lNa2RHR3piQXBKTWZWRVJR\
    TQo1NXorMFVqdS8zSFJhNFY3b3p2TGRPRE5HUURaeWNJU0l3VHBLbFR3RjBxazdCNjVhTUlJenQ1\
    dnhOK1lxYU1GClppZFRLNzcxNjdqNEExZ3F3MG45bjlybWVXUGRWZ3dudnRtVXp4Q1krNk05SXpK\
    TDI3eWpRUTV1WGQ3RVdMT3IKbndJREFRQUIKLS0tLS1FTkQgUFVCTElDIEtFWS0tLS0tCg=="

        try:
            # 计算hex值
            value_hex = a2b_hex(hsobj.hexdigest())
            # 加载公钥,填充格式OAEP
            uniqueKey = key.encode('utf-8')
            uniqueKeyorig = base64.b64decode(uniqueKey)  # 公钥文件
            rsa_pubkey = RSA.importKey(uniqueKeyorig)    # RSA公钥
            oaep_pub = PKCS1_OAEP.new(rsa_pubkey)        # OAEP填充
            # 加密数据
            encodemsg = oaep_pub.encrypt(value_hex)
            # 加密数据Base64转码
            enMsg = base64.b64encode(encodemsg)
        except ValueError:
            logger.error("Value error: %s.", hsobj.hexdigest())
        except TypeError:
            logger.error("RSA key has no private half.")

        try:
            bus = dbus.SystemBus()
            remote_object = bus.get_object("com.kylin.daq",
                                        "/com/kylin/daq")

            # you can either specify the dbus_interface in each call...
            reply_List = remote_object.UploadMessage(packageInfo,sendInfo,enMsg,dbus_interface = "com.kylin.daq.interface")
        except dbus.DBusException as e:
            logger.error("kylin-daq dbus call failed,",str(e))
        
        #存入tid
        if(len(reply_List) == 2):
            if len(reply_List[1]) != 0:
                sections = conf.sections()
                if "uploadMessage" in sections:
                    conf.set("uploadMessage","tid",reply_List[1])
                    conf.write(open("/usr/share/kylin-update-manager/tid.conf", "w"))

    
    
